;;;; -*- Mode: Lisp; Syntax: ANSI-Common-Lisp; Base: 10 -*-
;;;; *************************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          kmrcl-tests.lisp
;;;; Purpose:       kmrcl tests file
;;;; Author:        Kevin M. Rosenberg
;;;; Date Started:  Apr 2003
;;;;
;;;; $Id: tests.lisp,v 1.17 2003/06/07 05:45:14 kevin Exp $
;;;;
;;;; This file is Copyright (c) 2000-2002 by Kevin M. Rosenberg
;;;;
;;;; *************************************************************************

(in-package #:cl)
(defpackage #:kmrcl-tests
  (:use #:kmrcl #:cl #:rtest))
(in-package #:kmrcl-tests)

(rem-all-tests)


(deftest str.0 (substitute-chars-strings "" nil) "")
(deftest str.1 (substitute-chars-strings "abcd" nil) "abcd")
(deftest str.2 (substitute-chars-strings "abcd" nil) "abcd")
(deftest str.3 (substitute-chars-strings "abcd" '((#\j . "ef"))) "abcd")
(deftest str.4 (substitute-chars-strings "abcd" '((#\a . "ef"))) "efbcd")
(deftest str.5
    (substitute-chars-strings "abcd" '((#\a . "ef") (#\j . "ghi")))
  "efbcd")
(deftest str.6
    (substitute-chars-strings "abcd" '((#\a . "ef") (#\d . "ghi")))
  "efbcghi")

(deftest str.7 (escape-xml-string "") "")
(deftest str.8 (escape-xml-string "abcd") "abcd")
(deftest str.9 (escape-xml-string "ab&cd") "ab&amp;cd")
(deftest str.10 (escape-xml-string "ab&cd<") "ab&amp;cd&lt;")
(deftest str.12 (string-trim-last-character "") "")
(deftest str.13 (string-trim-last-character "a") "")
(deftest str.14 (string-trim-last-character "ab") "a")
(deftest str.15 (nstring-trim-last-character "") "")
(deftest str.16 (nstring-trim-last-character "a") "")
(deftest str.17 (nstring-trim-last-character "ab") "a")

(deftest str.18 (delimited-string-to-list "ab|cd|ef" #\|)
					  ("ab" "cd" "ef"))
(deftest str.19 (delimited-string-to-list "ab|cd|ef" #\| t)
					  ("ab" "cd" "ef"))
(deftest str.20 (delimited-string-to-list "") (""))
(deftest str.21 (delimited-string-to-list "" #\space t) (""))
(deftest str.22 (delimited-string-to-list "ab") ("ab"))
(deftest str.23 (delimited-string-to-list "ab" #\space t) ("ab"))
(deftest str.24 (delimited-string-to-list "ab|" #\|) ("ab" ""))
(deftest str.25 (delimited-string-to-list "ab|" #\| t) ("ab"))

(deftest apsl.1 (append-sublists '((a b) (c d))) (a b c d))
(deftest apsl.2 (append-sublists nil) nil)
(deftest apsl.3 (append-sublists '((a b))) (a b))
(deftest apsl.4 (append-sublists '((a))) (a))
(deftest apsl.5 (append-sublists '((a) (b) (c d (e f g)))) (a b c d (e f g)))

(deftest pss.0 (with-output-to-string (s) (print-separated-strings s "|" nil)) 
  "")

(deftest pss.1
    (with-output-to-string (s) (print-separated-strings s "|" '("ab")) )
  "ab")

(deftest pss.2
    (with-output-to-string (s) (print-separated-strings s "|" '("ab" "cd")))
    "ab|cd")

(deftest pss.3
    (with-output-to-string (s) (print-separated-strings s "|" '("ab" "cd") nil))
    "ab|cd")

(deftest pss.4
    (with-output-to-string (s)
      (print-separated-strings s "|" '("ab" "cd") nil nil))
    "ab|cd")

(deftest pss.5
    (with-output-to-string (s)
      (print-separated-strings s "|" '("ab" "cd") nil '("ef") nil))
    "ab|cd|ef")

(deftest css.0 (concat-separated-strings "|" nil) "")
(deftest css.1 (concat-separated-strings "|" nil nil) "")
(deftest css.2 (concat-separated-strings "|" '("ab")) "ab")
(deftest css.3 (concat-separated-strings "|" '("ab" "cd")) "ab|cd")
(deftest css.4 (concat-separated-strings "|" '("ab" "cd") nil) "ab|cd")
(deftest css.5 (concat-separated-strings "|" '("ab" "cd") nil '("ef")) "ab|cd|ef")

(deftest f.1 (filter #'(lambda (x) (when (oddp x) (* x x)))
		     '(0 1 2 3 4 5 6 7 8 9)) (1 9 25 49 81))
(deftest an.1 (appendnew '(a b c d) '(c c e f)) (a b c d e f))


(deftest pxml.1
  (xml-tag-contents "tag1" "<tag>Test</tag>")
  nil nil nil)

(deftest pxml.1o
  (kmrcl::xml-tag-contents-old "tag1" "<tag>Test</tag>")
  nil nil)

(deftest pxml.2
  (xml-tag-contents "tag" "<tag>Test</tag>")
  "Test" 15 nil)

(deftest pxml.3
  (xml-tag-contents "tag" "<tag  >Test</tag>")
  "Test" 17 nil)

(deftest pxml.4
    (xml-tag-contents "tag" "<tag a=\"b\"></tag>")
  "" 17 ("a=\"b\""))

(deftest pxml.5
    (xml-tag-contents "tag" "<tag a=\"b\" >Test</tag>")
  "Test" 22 ("a=\"b\""))

(deftest pxml.6
    (xml-tag-contents "tag" "<tag a=\"b\"  c=\"ab\">Test</tag>")
  "Test" 29 ("a=\"b\"" "c=\"ab\""))

(deftest pxml.7
    (xml-tag-contents "tag" "<taga a=\"b\"  c=\"ab\">Test</taga>")
  nil nil nil)

(deftest pxml.8
    (xml-tag-contents "tag" "<taga a=\"b\"  c=\"ab\">Test<tag>ab</tag></taga>")
  "ab" 37 nil)

(deftest pxml.9
    (xml-tag-contents "tag" "<taga a=\"b\"  c=\"ab\">Test<tag>ab</ag></taga>")
  nil nil nil)

(deftest fss.1 (fast-string-search "" "" 0 0 0) 0)
(deftest fss.2 (fast-string-search "" "abc" 0 0 2) 0)
(deftest fss.3 (fast-string-search "abc" "" 3 0 0) nil)
(deftest fss.4 (fast-string-search "abc" "abcde" 3 0 4) 0)
(deftest fss.5 (fast-string-search "abc" "012abcde" 3 0 7) 3)
(deftest fss.6 (fast-string-search "abc" "012abcde" 3 0 7) 3)
(deftest fss.7 (fast-string-search "abc" "012abcde" 3 3 7) 3)
(deftest fss.8 (fast-string-search "abc" "012abcde" 3 4 7) nil)
(deftest fss.9 (fast-string-search "abcde" "012abcde" 5 3 8) 3)
(deftest fss.9b (cl:search "abcde" "012abcde" :start2 3 :end2 8) 3)
(deftest fss.10 (fast-string-search "abcde" "012abcde" 5 3 7) nil)
(deftest fss.10b (cl:search "abcde" "012abcde" :start2 3 :end2 7) nil)

(deftest stlsd.1 (string-to-list-skip-delimiter "") ())
(deftest stlsd.2 (string-to-list-skip-delimiter "abc") ("abc"))
(deftest stlsd.3 (string-to-list-skip-delimiter "ab c") ("ab" "c"))
(deftest stlsd.4 (string-to-list-skip-delimiter "ab  c") ("ab" "c"))
(deftest stlsd.5 (string-to-list-skip-delimiter "ab   c") ("ab" "c"))
(deftest stlsd.6 (string-to-list-skip-delimiter "ab   c ") ("ab" "c"))
(deftest stlsd.7 (string-to-list-skip-delimiter "  ab   c  ") ("ab" "c"))
(deftest stlsd.8 (string-to-list-skip-delimiter "ab,,c" #\,) ("ab" "c"))
(deftest stlsd.9 (string-to-list-skip-delimiter "ab,,c,," #\,) ("ab" "c"))
(deftest stlsd.10 (string-to-list-skip-delimiter " ab") ("ab"))

;;; MOP Testing

(eval-when (:compile-toplevel :load-toplevel :execute)
  (when (find-package '#:kmr-mop)
    (pushnew :kmrtest-mop cl:*features*)))

#+kmrtest-mop
(progn
  (defclass credit-rating ()
    ((level :attributes (date-set time-set))
     (id :attributes (person-setting)))
    (:metaclass attributes-class))
  (defparameter cr nil)
  
  (defclass monitored-credit-rating (credit-rating)
    ((level :attributes (last-checked interval date-set))
     (cc :initarg :cc)
     (id :attributes (verified)))
    (:metaclass attributes-class))
  (defparameter mcr (make-instance 'monitored-credit-rating))

  (deftest attrib.mop.1
      (progn
	(setq cr (make-instance 'credit-rating))
	(slot-attribute cr 'level 'date-set))
      nil)

  (deftest attrib.mop.2
      (progn
	(setq cr (make-instance 'credit-rating))
	(setf (slot-attribute cr 'level 'date-set) "12/15/1990")
	(slot-attribute cr 'level 'date-set))
    "12/15/1990")

  (deftest attrib.mop.3
      (progn
	(setq mcr (make-instance 'monitored-credit-rating))
	(setf (slot-attribute mcr 'level 'date-set) "01/05/2002")
	(slot-attribute mcr 'level 'date-set))
    "01/05/2002")
  
  )   ;; kmrcl-mop

#+kmrtest-mop
(eval-when (:compile-toplevel :load-toplevel :execute)
  (setq cl:*features* (delete :kmrtest-mop cl:*features*)))
